#!/bin/sh

SCRIPT=common-srv-dat.sh
DIR_HTDOCS="/usr/local/apache2/htdocs"
DIR_HTDOCS_UKMUPDATES="$DIR_HTDOCS/ukmupdates"
DIR_CLI_UPDATES="ukmupdates"


echo "=============================="
echo " $SCRIPT started"
echo "=============================="

result()
{
  if [ $? -ne 0 ]; then
    echo "ERROR"
    exit 1
  fi
}

dir_create()
{
	local dir=$1

	if [ ! -d $dir ]; then
		echo "Creating directory $dir"
		mkdir -pv $dir
		result
	fi
}

dir_set_writable()
{
	local dir=$1

	echo "Setting writable access mode for $dir"
	chmod -v a+w $dir
	result
}

dir_create_writable()
{
	local dir=$1
	
	dir_create $dir
	dir_set_writable $dir
}


echo "Current directory is $(pwd)"

php_script="read-server-ini.php"
if [ ! -f $php_script ]; then
  echo "ERROR: File $php_script not found"
  exit 1
fi

php -n -d display_errors=On $php_script > ukmsrvdb_data
. ./ukmsrvdb_data
echo
echo "ukmserver database connection parameters: "
echo "  UKMSRVDBHOST=$UKMSRVDBHOST"
echo "  UKMSRVDBUSER=$UKMSRVDBUSER"
echo "  UKMSRVDBNAME=$UKMSRVDBNAME"
echo "  UKMSRVDBPASSWORD=$UKMSRVDBPASSWORD"
echo
rm -f ukmsrvdb_data


echo
echo "---- Client updates processing"
echo "Checking directory $DIR_CLI_UPDATES"
if [ -d $DIR_CLI_UPDATES ]; then
	echo "Directory $DIR_CLI_UPDATES found"
	
	dir_create_writable $DIR_HTDOCS_UKMUPDATES

	echo "Copying $DIR_CLI_UPDATES to $DIR_HTDOCS"
	cp -afv $DIR_CLI_UPDATES $DIR_HTDOCS
	result
	
	dir_set_writable $DIR_HTDOCS_UKMUPDATES
else
	echo "Directory $DIR_CLI_UPDATES not found - nothing to do"
fi
echo "---- Client updates processing"
echo

echo "=============================="
echo " $SCRIPT completed"
echo "=============================="
