<?php

/* возвращает структуру xml в виде массива
Array
(
	...
    [{i}] => Array
        (
            [tag] => {Имя_Тэга}
            [type] => {open - открывающий тэг|complete - полный тэг|close - закрывающий тэг|cdata - данные между открывающим и закрывающим тэгом}
            [level] => {уровень}
            [attributes] => Array
                (
					...
                    [{имя_атрибута}] => {значение атрибута}
                    ...
                )

            [value] => {значение между открывающим и закрывающим тэгами}
        )
    ...
)
*/

function parse_standard_xml($__xml)
{
	$xml_parser = xml_parser_create();

	xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, 0);
	xml_parse_into_struct($xml_parser, $__xml, $tags);
	xml_parser_free($xml_parser);

	return $tags;
}

function create_standard_xml($__tags)
{
	$content = "";
	foreach($__tags as $tag)
	{
		if ($tag['type'] == "complete" || $tag['type'] == "open")
		{
			$content .= "<".$tag['tag']." ";

			if(isset($tag['attributes']))
			{
				foreach($tag['attributes'] as $name => $value)
				{
					$content .= $name."=\"".htmlspecialchars($value)."\" ";
				}
			}

			if($tag['type'] == "complete")
			{
				if(isset($tag['value']))
				{
					$content .= ">".$tag['value']."</".$tag['tag'].">\n";
				}
				else
				{
					$content .= "></".$tag['tag'].">\n";
				}
			}
			else
			{
				$content .= ">";
			}
		}
		elseif ($tag["type"] == "close")
		{
			$content .= "</".$tag['tag'].">\n";
		}
		elseif ($tag["type"] == "cdata")
		{
			continue;
		}

		if($tag['type'] != "complete" && isset($tag['value']))
		{
			$content .= $tag['value'];
		}
	}
	return $content;
}


function upgrade_commands(&$__db, $command, $__change_version=false)
{
	$updater = "functor_command_".$command;
	print("update command ".$command."\n");
	print("\ttrm_in_keyboard\n");
	$buttons = $__db->getAll('select keyboard_layout_id, id, param from trm_in_keyboard where command = ? AND deleted = 0', array($command));
	foreach($buttons as $button)
	{
		$param = call_user_func($updater, $button['param']);
		if ($param != $button['param'])
		{
			$__db->query('update trm_in_keyboard set param = ?'.($__change_version ? ', version = 0' : '').' where keyboard_layout_id = ? and id = ? and deleted = 0',
				array($param, $button['keyboard_layout_id'], $button['id']));
		}
	}

	print("\ttrm_in_menu\n");
	$items = $__db->getAll('select terminal_menu_id, id, param from trm_in_menu where command = ? AND deleted = 0', array($command));
	foreach($items as $item)
	{
		$param = call_user_func($updater, $item['param']);
		if ($param != $item['param'])
		{
			$__db->query('update trm_in_menu set param = ?'.($__change_version ? ', version = 0' : '').' where terminal_menu_id = ? and id = ?',
				array($param, $item['terminal_menu_id'], $item['id']));
		}
	}

	print("\ttrm_in_screen\n");
	$panels = $__db->getAll("select id, content from trm_in_screen where deleted = 0");
	foreach($panels as $key => $panel)
	{
		$panel_tags_array = parse_standard_xml($panel['content']);
		foreach($panel_tags_array as $key1 => $tag)
		{
			if(isset($tag['attributes']['command']) && $tag['attributes']['command'] == $command)
			{
				$panel_tags_array[$key1]['attributes']['parameters'] = call_user_func($updater, $tag['attributes']['parameters']);
			}
		}
		$new_panel = create_standard_xml($panel_tags_array);
		$__db->query('update trm_in_screen set content = ?'.($__change_version ? ', version = 0' : '').' where global_id = 0 and id = ?',
			array($new_panel, $panel['id']));
	}
}

function upgrade_device(&$__db, $__device, $__change_version=false)
{
	$updater = "functor_device_".$__device;
	print("update device ".$__device."\n");
	print("\ttrm_in_config_hardware\n");
	$devices = $__db->getAll('select config_hardware_id, port, id, object_property from trm_in_config_hardware where _object = ? and deleted = 0', array($__device));
	foreach($devices as $device)
	{
		$object_property = call_user_func($updater, $device['object_property']);
		if ($object_property != $device['object_property'])
		{
			$__db->query('update trm_in_config_hardware set object_property = ?'.($__change_version ? ', version = 0' : '').' where config_hardware_id = ? and _object = ? and port = ? and id = ?',
				array($object_property, $device['config_hardware_id'], $__device, $device['port'], $device['id']));
		}
	}
}

function change_eq_param(&$__db, $__key_value, $__functor, $__change_version=false)
{
	print( "\n==== Updating equaring parameters efts = ".$__key_value);
	$params = $__db->getAll('SELECT store_id, id, param FROM trm_in_equarings WHERE efts = ? AND deleted = 0;', array($__key_value));
	$row_count = 0;
	foreach ($params as $param)
	{
		$res_param = $__functor($param['param']);
		
		if ($res_param != $param['param'])
		{
			$__db->query('UPDATE trm_in_equarings SET param = ?'.($__change_version ? ', version = 0' : '').' WHERE store_id = ? AND id = ?;', 
						array($res_param, $param['store_id'], $param['id']));
			$row_count++;
		}
	}
	print("\n==== Equaring parameters efts = ".$__key_value." updated. ".$row_count." records updated.");
}

//Функции для распарс. параметров
function explode_nm($string) {
			$arr=array();
			$wrd='';
			$ch='';
				for($i=0;$i<strlen($string);$i++) {
					$ch=substr($string,$i,1);
					if($ch!=',') {
						$wrd.=$ch;
					}
					else {
						if (substr($string,$i-1,1)!='\\') {
							$arr[]=$wrd;
							$wrd='';
						}
						else {
                            $wrd=substr($wrd,0,strlen($wrd)-1);
                            $wrd.=',';
						}
					}
					if (($i==(strlen($string)-1)))
					{
						$arr[]=$wrd;
                    }
				}
            return $arr;
		}

function explode_m($string) {
			$arr=explode_nm($string);
            foreach($arr as $k=>$v)
			{
				if(strstr($v,','))
				{
					$arr[$k]=explode(',',$v);
				}
			}
            return $arr;
		}

function implode_nm($arr) {
	foreach($arr as $key => $str) {
		$arr[$key] = str_replace( ',', '\\,', $str );
	}
	return implode( ',', $arr );
}


function set_max_ace(&$db)
{
	$max_ace_kbd = $db->getOne("select max(ace) from trm_in_keyboard");
	$max_ace_menu = $db->getOne("select max(ace) from trm_in_menu");
	$max_ace = max((int)$max_ace_kbd,(int)$max_ace_menu);

	$db->query("UPDATE sequences set id = ? where name = ?", array($max_ace, 'menu_kbd'));
	$db->commit();
}

function insert_command(&$db,$mystirious_id, $press_code, $command, $param, $permission_role_id)
{
    $is_this_code = $db->getOne("select id from trm_in_keyboard where keyboard_layout_id = ? and id = ? and deleted = 0"
								, array($mystirious_id, $press_code));
	if(!$is_this_code)
	{
		$ace = $db->getOne('SELECT next_sequence(?,0)', array('menu_kbd'));

		$db->query("REPLACE into trm_in_keyboard (keyboard_layout_id, id, ace, command, param, version, deleted) ".
					   "values (?,?,?,?,?,0,0)",
					   array(
							$mystirious_id
							,$press_code
							,$ace
							,$command
							,$param
							));

	   foreach($permission_role_id as $role)
	   {
		   $is_this_role = $db->getOne("select id from trm_in_role where id = ? and deleted = 0", array($role));
		   if($is_this_role)
		   {
			   $db->query("REPLACE into trm_in_role_permission (global_id,role_id,entry_id,version,deleted) ".
								"values (0, ?, ?, 0, 0)",
								array($role, $ace));
		   }
	   }
   }
}

function insert_char(&$db, $__table_name, $__myst_id, $press_code, $param)
{
	$db->query("REPLACE INTO ! VALUES (?, ?, 0, 612, ?, 0, 0)", array(
							$__table_name
							,$__myst_id
							,$press_code
							,$param
							));
}


function insert_service_functions(&$__db, $__table_name, $__myst_id, $press_code, $service_function)
{
	$__db->query("REPLACE INTO ! VALUES (?,?,0,?,'',0,0)", array(
							$__table_name
							,$__myst_id
							,$press_code
							,$service_function
							));
}

function upgrade_discount_common_params(&$db, $efts, $__change_version=false)
{
	$updater = "functor_upgrade_discount_common_params_".$efts;

	$discount_params = $db->getAll('select id, param 
								from trm_in_discount_types
								where efts=? and deleted=0',  array($efts) );

	foreach($discount_params as $disc_param)
	{
		$new_param = call_user_func($updater, $disc_param['param']);
		
		if($new_param != $disc_param['param'])
		{
			$db->query('update trm_in_discount_types set param = ?'.($__change_version ? ', version = 0' : '').' where id = ?',
					array($new_param, $disc_param['id']));	
		}
	}
}

function upgrade_auth_server_params(&$db, $efts, $both)
{
	$updater = "functor_upgrade_auth_server_params_".$efts;

	$servers_params = $db->getAll('select store_id, id, param 
								from trm_in_auth_servers
								where efts=? and deleted=0',  array($efts) );

	foreach($servers_params as $serv_param)
	{
		$new_param = call_user_func($updater, $serv_param['param']);
		
		if($new_param != $serv_param['param'])
		{
			$db->query('update trm_in_auth_servers set param = ?'. ($both ? '' : ', version = 0').' where id = ? and store_id = ?',
					array($new_param, $serv_param['id'], $serv_param['store_id']));	
		}
	}
}

function change_transferings_param(&$__db, $__key_value, $__functor, $__change_version=false)
{
	print( "\n==== Updating transferings parameters epts = ".$__key_value);
	$params = $__db->getAll('SELECT store_id, id, param FROM trm_in_transferings WHERE epts = ? AND deleted = 0;', array($__key_value));
	$row_count = 0;
	foreach ($params as $param)
	{
		$res_param = $__functor($param['param']);
		
		if ($res_param != $param['param'])
		{
			$__db->query('UPDATE trm_in_transferings SET param = ?'.($__change_version ? ', version = 0' : '').' WHERE store_id = ? AND id = ?;', 
						array($res_param, $param['store_id'], $param['id']));
			$row_count++;
		}
	}
	print("\n==== transferings parameters epts = ".$__key_value." updated. ".$row_count." records updated.\n");
}

function change_payment_param(&$__db, $__efts, $__functor)
{
	print( "\n==== Updating payment parameters with efts = ".$__efts);
	$params = $__db->getAll('SELECT p.store_id, p.id, p.param FROM trm_in_payments p 
							INNER JOIN trm_in_equarings e ON p.store_id = e.store_id AND p.equaring_id = e.id AND e.efts = ? AND e.deleted = 0 
							WHERE p.deleted = 0;', array($__efts));
	$row_count = 0;
	foreach ($params as $param)
	{
		$res_param = $__functor($param['param']);
		
		if ($res_param != $param['param'])
		{
			$__db->query('UPDATE trm_in_payments SET param = ?, version = 0 WHERE store_id = ? AND id = ?;', 
						array($res_param, $param['store_id'], $param['id']));
			$row_count++;
		}
	}
	print("\n==== Payment parameters with efts = ".$__efts." updated. ".$row_count." records updated.");
}

function add_column(&$__db, $__table, $__col, $__description)
{
	$exists = $__db->getAll('SHOW COLUMNS FROM ! WHERE field = ?', array($__table, $__col));
	$res = 0;
	if (!isset($exists) || empty($exists))
	{
		print ("==== adding column ".$__col." to ".$__table." ====\n");
		$res = $__db->query('ALTER TABLE ! ADD COLUMN ! !;', 
					array($__table, $__col, $__description));
	}
	else
	{
		print ("==== column ".$__col." already exists in ".$__table." ====\n");
	}
	print($res." updated\n");
	return $res;
}

function del_notuse_auth_server(&$__db, $__serv_efts, $__command_efts, $__discount_efts)
{
	$auth_servers = array();
	$commands = $__db->getAll("SELECT param FROM trm_in_keyboard where deleted = 0 AND command in ( '" . implode($__command_efts, "', '") . "' )");
	foreach ($commands as $command)
	{
		if (!empty($command['param']))
			$auth_servers[] = $command['param']; 
	}

	$discounts = $__db->getAll("SELECT param FROM trm_in_discount_types where deleted = 0 AND efts in ( '" . implode($__discount_efts, "', '") . "' )");
	foreach ($discounts as $discount)
	{
		if (!empty($discount['param']))
		{			
			$discount_tags_array = parse_standard_xml($discount['param']);
			$find_equaring = false;
			foreach($discount_tags_array as $key => $tag)
			{
				if($tag['tag'] == 'equaring_id' && $tag['type'] == 'open')
				{
					$find_equaring = true;
				}
				if($tag['tag'] == 'value' && $tag['type'] == 'complete' && $find_equaring)
				{
					if (isset($tag['value']))
					{
						$auth_servers[] = $tag['value']; 
					} 
					$find_equaring = false;
				}
			}
		}
	}
	$__db->query("DELETE FROM trm_in_auth_servers WHERE efts = ? AND id NOT IN  ( '" . implode($auth_servers, "', '") . "' )", $__serv_efts);
}

function add_converter_params(&$__db, $cnv_code, $param_name, $param_value, $parent_tag_name, $use_value_tag)
{
	$converters = $__db->getAll("SELECT cnv_id, cnv_pr_xml FROM srv_converters where cnv_code = '". $cnv_code ."';");
	foreach ($converters as $converter)
	{
		if (!empty($converter['cnv_pr_xml']))
		{
			$converter_tags_array = parse_standard_xml($converter['cnv_pr_xml']);
			$param_fount = false;
			$params_open_index = -1;
			$cur_level = -1;
			$parent_lavel = -1;
			$res = array();
			foreach($converter_tags_array as $key => $tag)
			{
				if(($tag['tag'] == 'params') && ($tag['type'] == 'open'))
				{
					$params_open_index = $key;
					$parent_lavel = $tag['level'];
					$cur_level = $parent_lavel + 1;
				}
				if ($tag['tag'] == $param_name)
				{
					$param_fount = true;
					break;
				}
			}
			if (!$param_fount && $params_open_index > 0)
			{
				$new_param_tag = array('tag' => $param_name,
										'type' => 'complete',
										'level' => $cur_level,
										'value' => $param_value,);
				if ($use_value_tag == true)
				{
					$new_param_tag['type'] = 'open';
					$new_param_tag['value'] = '';
					
					$cur_value_level = $cur_level + 1;
					$new_param_tag_value = array('tag' => 'value',
												'type' => 'complete',
												'level' => $cur_value_level,
												'value' => $param_value,);
					$new_param_tag_close = array('tag' => $param_name,
												'type' => 'close',
												'level' => $cur_level,
												'value' => '',);
				}
				$param_end = array('tag' => $parent_tag_name,
									'value' => '',
									'type' => 'cdata',
									'level' => $parent_lavel,);
				$res = array_slice($converter_tags_array, 0, $params_open_index + 1, true);
				array_push($res, $new_param_tag);
				if ($use_value_tag == true)
				{
					array_push($res, $new_param_tag_value);
					array_push($res, $new_param_tag_close);
				}
//				array_push($res, $param_end);
				$tail = array_slice($converter_tags_array, $params_open_index + 1, count($converter_tags_array) - 1);
				foreach($tail as $key => $tag)
				{
					array_push($res, $tag);
				}
				$new_params = '<?xml version="1.0" encoding="UTF-8"?>\n<!DOCTYPE boost_serialization>\n';
				
				$new_params .= create_standard_xml($res);
				$__db->query("UPDATE srv_converters SET cnv_pr_xml = '". $new_params ."' WHERE cnv_id = '". $converter['cnv_id'] ."';");
			}
		}
	}
}

function add_discount_xml_param(&$__db, $discount_efts, $param_name, $param_value, $parent_tag_name, $use_value_tag)
{
	$discounts = $__db->getAll("SELECT id, param FROM trm_in_discount_types where efts = ". $discount_efts .";");
	foreach ($discounts as $discount)
	{
		if (!empty($discount['param']))
		{
			$discount_tags_array = parse_standard_xml($discount['param']);
			$param_fount = false;
			$params_open_index = -1;
			$cur_level = -1;
			$parent_lavel = -1;
			$res = array();
			foreach($discount_tags_array as $key => $tag)
			{
				if(($tag['tag'] == $parent_tag_name) && ($tag['type'] == 'open'))
				{
					$params_open_index = $key;
					$parent_lavel = $tag['level'];
					$cur_level = $parent_lavel + 1;
				}
				if ($tag['tag'] == $param_name)
				{
					$param_fount = true;
					break;
				}
			}
			if (!$param_fount && $params_open_index >= 0)
			{
				$new_param_tag = array('tag' => $param_name,
										'type' => 'complete',
										'level' => $cur_level,
										'value' => $param_value,);
				if ($use_value_tag == true)
				{
					$new_param_tag['type'] = 'open';
					$new_param_tag['value'] = '';
					
					$cur_value_level = $cur_level + 1;
					$new_param_tag_value = array('tag' => 'value',
												'type' => 'complete',
												'level' => $cur_value_level,
												'value' => $param_value,);
					$new_param_tag_close = array('tag' => $param_name,
												'type' => 'close',
												'level' => $cur_level,
												'value' => '',);
				}
				$param_end = array('tag' => $parent_tag_name,
									'value' => '',
									'type' => 'cdata',
									'level' => $parent_lavel,);
				$res = array_slice($discount_tags_array, 0, $params_open_index + 1, true);
				array_push($res, $new_param_tag);
				if ($use_value_tag == true)
				{
					array_push($res, $new_param_tag_value);
					array_push($res, $new_param_tag_close);
				}
				array_push($res, $param_end);
				$tail = array_slice($discount_tags_array, $params_open_index + 1, count($discount_tags_array) - 1);
				foreach($tail as $key => $tag)
				{
					array_push($res, $tag);
				}
				$new_params = create_standard_xml($res);
				$__db->query("UPDATE trm_in_discount_types SET param = '". $new_params ."' WHERE id = '". $discount['id'] ."';");
			}
		}
	}
}
?>
